module.exports = {
   "env" : {
      "browser" : true,
      "es2020"  : true
   },
   "extends"       : "eslint:recommended",
   "parserOptions" : {
      "ecmaVersion" : 11
   },
   "ignorePatterns" : ["**/build/**/*.js"],
   "rules"          : {

      "no-unused-vars": [
          "error",
          {
             // unused arguments are ok
             "args": "none"
          }
       ],
   
      "operator-linebreak": ["error", "before"],
      
      /* Using double quote (") for strings */
      "quotes" : [
         "error",
         "double"
      ],
      // References
      /* This ensures that you can’t reassign your references,
       * which can lead to bugs and difficult to comprehend code
       */
      "prefer-const"             : "off",
      /* Prefer using block-scope declaration (let, const),
       * Why: https://evertpot.com/javascript-let-const/
       */
      "no-var"                   : "error",
      // Objects
      /* No difference to "new Object()" or "new Array()"
       * but more beautiful
       * Prefer const x = {}; than const x = new Object();
       */
      "no-new-object"            : "error",
      "no-array-constructor"     : "error",
      /* example: newArray = array.map(() => foo())
       * if no returned value, newArray could contain just undefined
       * It leads to bugs
       */
      "array-callback-return"    : "error",
      // Functions
      /* Creating a function in this way evaluates a string similarly to eval()
       * which opens vulnerabilities.
       */
      "no-new-func"              : "error",
      /* Example:
       * const x = function() {};
       */
      "space-before-blocks"      : "warn",
      // Arrow function
      /* Avoid bad readable snippets like this
       * const itemHeight = (item) => item.height <= 256 ? item.largeSize : item.smallSize;
       */
      "no-confusing-arrow"       : "error",
      /* Minimize diff churn when adding or removing arguments
       * Ex: bad    array.map(x => x * x)
       *     good   array.((x) => x * x)
       */
      "arrow-parens"             : "warn",
      /*
       * Don't write something like
       * array.map((x)
       *            => {
       *               });
       * Prefer array.map((x) => {
       *
       *        });
       */
      "implicit-arrow-linebreak" : "error",
      /* By using multiline blocks with if, else
       * else should be on the same line as if block's
       * closing brace
       */
      "brace-style"              : "warn",
      /* If a "if" block executes a return statement,
       * the else block is unnecessary
       * Example: Bad
       * if (a == b) {
       *     return true;
       * } else {
       *     return false;
       * }
       * Good
       * if (a == b) {
       *     return true;
       * }
       * return false;
       */
      "no-else-return"           : "warn",
      /* Prefer using === (type-save) than just ==
       * More explanation: https://eslint.org/docs/rules/eqeqeq
       */
      "eqeqeq"                   : "error",
      /* Start all comment with a space => easier to read */
      "spaced-comment"           : "warn",
      /* use tabs (4 spaces) */
      "indent"                   : [
         "warn",
         3,
         { "VariableDeclarator": "first" }
      ],
      /* Put space before block's begin */
      "keyword-spacing"         : "warn",
      /* Set off operators with spaces */
      "space-infix-ops"         : "warn",
      "no-multiple-empty-lines" : "warn",
      /* no space in parens */
      "space-in-parens"         : "warn",
      /* Example: prefer: [1, 2, 3] than [ 1, 2, 3 ] */
      "array-bracket-spacing"   : "warn",
      /* prefer: { x: 1 } than {x: 1} */
      "object-curly-spacing"    : [
         "warn",
         "always"
      ],
      /* Avoid space before commas and require space after commas
       * [1, 2] better than [1 , 2]
       */
      "comma-spacing"     : "warn",
      /* func() better than func () */
      "func-call-spacing" : "warn",
      /* { foo: 1 } better than { foo:1 } */
      "key-spacing"       : ["warn", {
         "singleLine" : {
            "beforeColon" : false,
            "afterColon"  : true
         },
         "multiLine" : {
            "beforeColon" : true,
            "afterColon"  : true,
            "align"       : "colon"
         }
      }],
      /* Prefer { foo: 1,
       *          bar: 2}
       * than { foo: 1
       *        , bar: 2}
       */
      "comma-style" : [
         "warn",
         "last"
      ],
      /* Why: https://github.com/airbnb/javascript#semicolons--required */
      "semi" : [
         "error",
         "always"
      ],
      // Naming convention
      "camelcase"            : "warn",
      /* Use pascal case when naming constructors or classes */
      "new-cap"              : "warn",
      /* Why: https://github.com/airbnb/javascript#naming--leading-underscore */
      "no-underscore-dangle" : "warn",
      /* Template string gives you a readable, concise syntax
       * with proper newlines and string interpolation features.
       * Good: const hello = `Hello. My name is ${name}`
       * Bad: const hello = "Hello. My name is" + name
      */
      "prefer-template"      : "warn",
      "max-len"              : [
         "warn",
         {
            "code"                   : 140,
            "ignoreStrings"          : true,
            "ignoreUrls"             : true,
            "ignoreComments"         : true,
            "ignoreTrailingComments" : true,
            "ignoreTemplateLiterals" : true
         }
      ],
      "curly" : ["warn", "all"],
      
      // use me as valid this alias
      "consistent-this": ["WARN", "me"],
      
      // enforce newline in objects with more than one property
      "object-curly-newline": ["error", {
         "consistent": true
      }]
   },
   "globals" : {
      "BpcCommon" : "readonly",
      "Ext"       : "readonly"
   }
};
